@@ -1,8 +1,9 @@
 #!/bin/sh
+set -e
 
 # Build a cvs2svn distribution.
 
-VERSION=1.3.0
+VERSION=1.3.1
 WC_REV=`svnversion -n .`
 DIST_BASE=cvs2svn-${VERSION}
 DIST_FULL=${DIST_BASE}.tar.gz
@@ -16,28 +17,19 @@ if echo ${WC_REV} | grep -q -e '[^0-9]'; then
 fi
 
 # Clean up anything that might have been left from a previous run.
-rm -rf dist MANIFEST cvs2svn-${VERSION} ${DIST_FULL}
+rm -rf dist MANIFEST ${DIST_BASE} ${DIST_FULL} cvs2svn-dist.sh-backup
+
+# Tweak cvs2svn to embed the proper version number.
+mv cvs2svn cvs2svn-dist.sh-backup
+sed -e "s/^VERSION = .*/VERSION = '${VERSION}'/" \
+  < cvs2svn-dist.sh-backup > cvs2svn
 
 # Build the dist, Python's way.
 ./setup.py sdist
 mv dist/${DIST_FULL} .
 
-# Unfortunately, building the dist Python's way doesn't seem to give
-# us an obvious method for including subdirs.  So, we rewire it!
-tar zxf ${DIST_FULL}
-rm ${DIST_FULL}
-svn export -q test-data ${DIST_BASE}/test-data
-svn export -q svntest ${DIST_BASE}/svntest
-svn export -q www ${DIST_BASE}/www
-# Oh, and while we're at it, let's fix cvs2svn's version number.
-sed -e "s/^VERSION = .*/VERSION = '${VERSION}'/" < cvs2svn > cvs2svn.tmp
-mv cvs2svn.tmp ${DIST_BASE}/cvs2svn
-chmod a+x ${DIST_BASE}/cvs2svn
-cp cvs2svn.1 ${DIST_BASE}
-tar zcf ${DIST_FULL} ${DIST_BASE}
-rm -rf ${DIST_BASE}
-
 # Clean up after this run.
+mv cvs2svn-dist.sh-backup cvs2svn
 rm -rf dist MANIFEST
 
 # We're outta here.
