@@ -2,11 +2,11 @@
 #
 #  actions.py:  routines that actually run the svn client.
 #
-#  Subversion is a tool for revision control. 
+#  Subversion is a tool for revision control.
 #  See http://subversion.tigris.org for more information.
-#    
+#
 # ====================================================================
-# Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2006 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -16,7 +16,7 @@
 #
 ######################################################################
 
-import os.path, shutil, string, re, sys
+import os.path, shutil, string, re, sys, errno
 
 import main, tree, wc  # general svntest routines in this module.
 from svntest import Failure, SVNAnyOutput
@@ -47,7 +47,7 @@ class SVNExpectedStderr(SVNUnexpectedOutput):
   pass
 
 class SVNIncorrectDatatype(SVNUnexpectedOutput):
-  """Exception raised if invalid input is passed to the 
+  """Exception raised if invalid input is passed to the
   run_and_verify_* API"""
   pass
 
@@ -70,7 +70,7 @@ def guarantee_greek_repository(path):
   # If there's no pristine repos, create one.
   if not os.path.exists(main.pristine_dir):
     main.create_repos(main.pristine_dir)
-    
+
     # dump the greek tree to disk.
     main.greek_state.write_to_disk(main.greek_dump_dir)
 
@@ -135,7 +135,7 @@ def run_and_verify_svnversion(message, wc_dir, repo_url,
   "Run svnversion command and check its output"
 
   out, err = main.run_svnversion(wc_dir, repo_url)
-  
+
   if type(expected_stdout) is type([]):
     compare_and_display_lines(message, 'STDOUT', expected_stdout, out)
   elif expected_stdout == SVNAnyOutput:
@@ -144,17 +144,17 @@ def run_and_verify_svnversion(message, wc_dir, repo_url,
       raise SVNExpectedStdout
   elif expected_stdout is not None:
     raise SVNIncorrectDatatype("Unexpected specification for stdout data")
-  
+
   if type(expected_stderr) is type([]):
     compare_and_display_lines(message, 'STDERR', expected_stderr, err)
   elif expected_stderr == SVNAnyOutput:
     if len(err) == 0:
       if message is not None: print message
       raise SVNExpectedStderr
-  elif expected_stderr is not None:
+  else:
     raise SVNIncorrectDatatype("Unexpected specification for stderr data")
   return out, err
-  
+
 
 def run_and_verify_svn(message, expected_stdout, expected_stderr, *varargs):
   """Invokes main.run_svn with *VARARGS, return stdout and stderr as
@@ -166,10 +166,15 @@ def run_and_verify_svn(message, expected_stdout, expected_stderr, *varargs):
      - If it is a single string, invoke match_or_fail() on MESSAGE,
        the expected output, and the actual output.
 
-     - If it is None, do nothing with it.
+  If EXPECTED_STDOUT is None, do not check stdout.
+  EXPECTED_STDERR may not be None.
 
   If a comparison function fails, it will raise an error."""
   ### TODO catch and throw particular exceptions from above
+
+  if expected_stderr is None:
+    raise SVNIncorrectDatatype("expected_stderr must not be None")
+
   want_err = None
   if expected_stderr is not None and expected_stderr is not []:
     want_err = 1
@@ -177,8 +182,8 @@ def run_and_verify_svn(message, expected_stdout, expected_stderr, *varargs):
   out, err = main.run_svn(want_err, *varargs)
 
   for (expected, actual, output_type, raisable) in (
-      (expected_stdout, out, 'stdout', SVNExpectedStdout),
-      (expected_stderr, err, 'stderr', SVNExpectedStderr)):
+      (expected_stderr, err, 'stderr', SVNExpectedStderr),
+      (expected_stdout, out, 'stdout', SVNExpectedStdout)):
     if type(expected) is type([]):
       compare_and_display_lines(message, output_type.upper(), expected, actual)
     elif type(expected) is type(''):
@@ -296,7 +301,7 @@ def verify_update(actual_output, wc_dir_name,
                   singleton_handler_b, b_baton,
                   check_props):
   """Verify update of WC_DIR_NAME.
-  
+
   The subcommand output (found in ACTUAL_OUTPUT) will be verified
   against OUTPUT_TREE, and the working copy itself will be verified
   against DISK_TREE.  If optional STATUS_OUTPUT_TREE is given, then
@@ -374,7 +379,7 @@ def run_and_verify_update(wc_dir_name,
     for line in errput:
       match = rm.search(line)
       if match:
-        return 
+        return
     raise main.SVNUnmatchedError
 
   mytree = tree.build_tree_from_checkout (output)
@@ -395,8 +400,33 @@ def run_and_verify_merge(dir, rev1, rev2, url,
                          check_props = 0,
                          dry_run = 1,
                          *args):
+  """Run 'svn merge -rREV1:REV2 URL DIR'."""
+  if args:
+    run_and_verify_merge2(dir, rev1, rev2, url, None, output_tree, disk_tree,
+                          status_tree, skip_tree, error_re_string,
+                          singleton_handler_a, a_baton, singleton_handler_b,
+                          b_baton, check_props, dry_run, *args)
+  else:
+    run_and_verify_merge2(dir, rev1, rev2, url, None, output_tree, disk_tree,
+                          status_tree, skip_tree, error_re_string,
+                          singleton_handler_a, a_baton, singleton_handler_b,
+                          b_baton, check_props, dry_run)
 
-  """Run 'svn merge -rREV1:REV2 URL DIR'
+
+def run_and_verify_merge2(dir, rev1, rev2, url1, url2,
+                          output_tree, disk_tree, status_tree, skip_tree,
+                          error_re_string = None,
+                          singleton_handler_a = None,
+                          a_baton = None,
+                          singleton_handler_b = None,
+                          b_baton = None,
+                          check_props = 0,
+                          dry_run = 1,
+                          *args):
+  """Run 'svn merge URL1@REV1 URL2@REV2 DIR' if URL2 is not None
+  (for a three-way merge between URLs and WC).
+
+  If URL2 is None, run 'svn merge -rREV1:REV2 URL1 DIR'.
 
   If ERROR_RE_STRING, the merge must exit with error, and the error
   message must match regular expression ERROR_RE_STRING.
@@ -410,12 +440,12 @@ def run_and_verify_merge(dir, rev1, rev2, url,
   SINGLETON_HANDLER_A and SINGLETON_HANDLER_B will be passed to
   tree.compare_trees - see that function's doc string for more
   details.
-  
+
   If CHECK_PROPS is set, then disk comparison will examine props.
 
   If DRY_RUN is set then a --dry-run merge will be carried out first and
   the output compared with that of the full merge.
-  
+
   Returns if successful, raises on failure."""
 
   if isinstance(output_tree, wc.State):
@@ -427,7 +457,11 @@ def run_and_verify_merge(dir, rev1, rev2, url,
   if isinstance(skip_tree, wc.State):
     skip_tree = skip_tree.old_tree()
 
-  merge_command = ('merge', '-r', rev1 + ':' + rev2, url, dir)
+  if url2:
+    merge_command = ("merge", url1 + "@" + str(rev1),url2 + "@" + str(rev2),
+                     dir)
+  else:
+    merge_command = ("merge", "-r", str(rev1) + ":" + str(rev2), url1, dir)
 
   if dry_run:
     pre_disk = tree.build_tree_from_wc(dir)
@@ -442,7 +476,7 @@ def run_and_verify_merge(dir, rev1, rev2, url,
       print "Dry-run merge altered working copy"
       print "============================================================="
       raise
-      
+
 
   # Update and make a tree of the output.
   merge_command = merge_command + args
@@ -563,7 +597,11 @@ def run_and_verify_commit(wc_dir_name, output_tree, status_output_tree,
     status_output_tree = status_output_tree.old_tree()
 
   # Commit.
-  output, errput = main.run_svn(error_re_string, 'ci', '-m', 'log msg', *args)
+  output, errput = main.run_svn(error_re_string, 'ci',
+                                '--username', main.wc_author,
+                                '--password', main.wc_passwd,
+                                '-m', 'log msg',
+                                *args)
 
   if (error_re_string):
     rm = re.compile(error_re_string)
@@ -579,7 +617,7 @@ def run_and_verify_commit(wc_dir_name, output_tree, status_output_tree,
   lastline = ""
   if len(output):
     lastline = string.strip(output.pop())
-    
+
     cm = re.compile("(Committed|Imported) revision [0-9]+.")
     match = cm.search(lastline)
     if not match:
@@ -601,10 +639,10 @@ def run_and_verify_commit(wc_dir_name, output_tree, status_output_tree,
     if not match:
       # whoops, it was important output, put it back.
       output.append(lastline)
-    
+
   # Convert the output into a tree.
   mytree = tree.build_tree_from_commit (output)
-    
+
   # Verify actual output against expected output.
   try:
     tree.compare_trees (mytree, output_tree)
@@ -612,7 +650,7 @@ def run_and_verify_commit(wc_dir_name, output_tree, status_output_tree,
       display_trees("Output of commit is unexpected.",
                     "OUTPUT TREE", output_tree, mytree)
       raise
-    
+
   # Verify via 'status' command too, if possible.
   if status_output_tree:
     run_and_verify_status(wc_dir_name, status_output_tree)
@@ -634,7 +672,7 @@ def run_and_verify_status(wc_dir_name, output_tree,
   if isinstance(output_tree, wc.State):
     output_tree = output_tree.old_tree()
 
-  output, errput = main.run_svn (None, 'status', '-v', '-u', '-q', 
+  output, errput = main.run_svn (None, 'status', '-v', '-u', '-q',
                                  '--username', main.wc_author,
                                  '--password', main.wc_passwd,
                                  wc_dir_name)
@@ -680,6 +718,50 @@ def run_and_verify_unquiet_status(wc_dir_name, output_tree,
     tree.compare_trees (mytree, output_tree)
 
 
+def run_and_verify_diff_summarize(output_tree, error_re_string = None,
+                                  singleton_handler_a = None,
+                                  a_baton = None,
+                                  singleton_handler_b = None,
+                                  b_baton = None,
+                                  *args):
+  """Run 'diff --summarize' with the arguments *ARGS.
+  If ERROR_RE_STRING, the command must exit with error, and the error
+  message must match regular expression ERROR_RE_STRING.
+
+  Else if ERROR_RE_STRING is None, the subcommand output will be
+  verified against OUTPUT_TREE.  SINGLETON_HANDLER_A and
+  SINGLETON_HANDLER_B will be passed to tree.compare_trees - see that
+  function's doc string for more details.  Returns on success, raises
+  on failure."""
+
+  if isinstance(output_tree, wc.State):
+    output_tree = output_tree.old_tree()
+
+  output, errput = main.run_svn (None, 'diff', '--summarize',
+                                 '--username', main.wc_author,
+                                 '--password', main.wc_passwd,
+                                 *args)
+
+  if (error_re_string):
+    rm = re.compile(error_re_string)
+    for line in errput:
+      match = rm.search(line)
+      if match:
+        return
+    raise main.SVNUnmatchedError
+
+  mytree = tree.build_tree_from_diff_summarize (output)
+
+  # Verify actual output against expected output.
+  try:
+    tree.compare_trees (mytree, output_tree,
+                        singleton_handler_a, a_baton,
+                        singleton_handler_b, b_baton)
+  except tree.SVNTreeError:
+    display_trees(None, 'DIFF OUTPUT TREE', output_tree, mytree)
+    raise
+
+
 ######################################################################
 # Displaying expected and actual output
 
@@ -729,11 +811,10 @@ def match_or_fail(message, label, expected, actual):
   """Make sure that regexp EXPECTED matches at least one line in list ACTUAL.
   If no match, then print MESSAGE (if it's not None), followed by
   EXPECTED and ACTUAL, both labeled with LABEL, and raise SVNLineUnequal."""
-  matched = None
   for line in actual:
     if re.match(expected, line):
-      matched = 1
-  if not matched:
+      break
+  else:
     display_lines(message, label, expected, actual, 1)
     raise main.SVNLineUnequal
 
@@ -742,7 +823,7 @@ def match_or_fail(message, label, expected, actual):
 
 
 # This allows a test to *quickly* bootstrap itself.
-def make_repo_and_wc(sbox):
+def make_repo_and_wc(sbox, create_wc = True):
   """Create a fresh repository and checkout a wc from it.
 
   The repo and wc directories will both be named TEST_NAME, and
@@ -756,20 +837,27 @@ def make_repo_and_wc(sbox):
   # Create (or copy afresh) a new repos with a greek tree in it.
   guarantee_greek_repository(sbox.repo_dir)
 
-  # Generate the expected output tree.
-  expected_output = main.greek_state.copy()
-  expected_output.wc_dir = sbox.wc_dir
-  expected_output.tweak(status='A ', contents=None)
+  if create_wc:
+    # Generate the expected output tree.
+    expected_output = main.greek_state.copy()
+    expected_output.wc_dir = sbox.wc_dir
+    expected_output.tweak(status='A ', contents=None)
 
-  # Generate an expected wc tree.
-  expected_wc = main.greek_state
-
-  # Do a checkout, and verify the resulting output and disk contents.
-  run_and_verify_checkout(main.current_repo_url,
-                          sbox.wc_dir,
-                          expected_output,
-                          expected_wc)
+    # Generate an expected wc tree.
+    expected_wc = main.greek_state
 
+    # Do a checkout, and verify the resulting output and disk contents.
+    run_and_verify_checkout(main.current_repo_url,
+                            sbox.wc_dir,
+                            expected_output,
+                            expected_wc)
+  else:
+    # just make sure the parent folder of our working copy is created
+    try:
+      os.mkdir(main.general_wc_dir)
+    except OSError, err:
+      if err.errno != errno.EEXIST:
+        raise
 
 # Duplicate a working copy or other dir.
 def duplicate_dir(wc_name, wc_copy_name):
@@ -778,7 +866,7 @@ def duplicate_dir(wc_name, wc_copy_name):
 
   main.safe_rmtree(wc_copy_name)
   shutil.copytree(wc_name, wc_copy_name)
-  
+
 
 
 def get_virginal_state(wc_dir, rev):
@@ -803,19 +891,19 @@ def lock_admin_dir(wc_dir):
   path = os.path.join(wc_dir, main.get_admin_name(), 'lock')
   main.file_append(path, "stop looking!")
 
-def enable_revprop_changes(repos_dir):
+def enable_revprop_changes(repo_dir):
   """Enable revprop changes in a repository REPOS_DIR by creating a
 pre-revprop-change hook script and (if appropriate) making it executable."""
-  if os.name == 'posix':
-    hook = os.path.join(repos_dir,
-                        'hooks', 'pre-revprop-change')
-    main.file_append(hook, "#!/bin/sh\n\nexit 0\n")
-    os.chmod(hook, 0755)
-  elif sys.platform == 'win32':
-    hook = os.path.join(repos_dir,
-                        'hooks', 'pre-revprop-change.bat')
-    main.file_append(hook, "@exit 0\n")
 
+  hook_path = main.get_pre_revprop_change_hook_path (repo_dir)
+  main.create_python_hook_script (hook_path, 'import sys; sys.exit(0)')
 
+def create_failing_post_commit_hook(repo_dir):
+  """Disable commits in a repository REPOS_DIR by creating a post-commit hook
+script which always reports errors."""
 
+  hook_path = main.get_post_commit_hook_path (repo_dir)
+  main.create_python_hook_script (hook_path, 'import sys; '
+    'sys.stderr.write("Post-commit hook failed"); '
+    'sys.exit(1)')
 ### End of file.
