@@ -2,11 +2,11 @@
 #
 #  tree.py: tools for comparing directory trees
 #
-#  Subversion is a tool for revision control. 
+#  Subversion is a tool for revision control.
 #  See http://subversion.tigris.org for more information.
-#    
+#
 # ====================================================================
-# Copyright (c) 2001 CollabNet.  All rights reserved.
+# Copyright (c) 2001, 2006 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -116,9 +116,9 @@ class SVNTypeMismatch(SVNTreeError):
 #   - in the 'svn status' use-case (which is always run with the -v
 #     (--verbose) flag), each line of output contains a working revision
 #     number and a two-letter status code similar to the 'svn co/up'
-#     case.  The repository revision is also printed.  All of this
-#     information is stored in attributes named 'wc_rev', 'status', and
-#     'repos_rev', respectively.
+#     case.  This information is stored in attributes named 'wc_rev'
+#     and 'status'.  The repository revision is also printed, but it
+#     is ignored.
 
 #   - in the working-copy use-case, the att-hash is ignored.
 
@@ -167,10 +167,10 @@ class SVNTreeNode:
         break
     if child_already_exists == 0:
       self.children.append(newchild)
-      newchild.path = os.path.join (self.path, newchild.name)      
+      newchild.path = os.path.join (self.path, newchild.name)
 
     # If you already have the node,
-    else:      
+    else:
       if newchild.children is None:
         # this is the 'end' of the chain, so copy any content here.
         a.contents = newchild.contents
@@ -181,25 +181,35 @@ class SVNTreeNode:
         # try to add dangling children to your matching node
         for i in newchild.children:
           a.add_child(i)
-      
+
 
   def pprint(self):
+    "Pretty-print the meta data for this node."
     print " * Node name:  ", self.name
     print "    Path:      ", self.path
-    print "    Contents:  ", self.contents
+    mime_type = self.props.get("svn:mime-type")
+    if not mime_type or mime_type.startswith("text/"):
+      if self.children is not None:
+        print "    Contents:   N/A (node is a directory)"
+      else:
+        print "    Contents:  ", self.contents
+    else:
+      print "    Contents:   %d bytes (binary)" % len(self.contents)
     print "    Properties:", self.props
     print "    Attributes:", self.atts
     ### FIXME: I'd like to be able to tell the difference between
     ### self.children is None (file) and self.children == [] (empty
-    ### diretory), but it seems that most places that construct
+    ### directory), but it seems that most places that construct
     ### SVNTreeNode objects don't even try to do that.  --xbc
+    ###
+    ### See issue #1611 about this problem.  -kfogel
     if self.children is not None:
       print "    Children:  ", len(self.children)
     else:
-      print "    Children: is a file."
+      print "    Children:   N/A (node is a file)"
 
 # reserved name of the root of the tree
-root_node_name = "__SVN_ROOT_NODE" 
+root_node_name = "__SVN_ROOT_NODE"
 
 
 # helper func
@@ -238,7 +248,7 @@ def node_is_greater(a, b):
 # Helper for compare_trees
 def compare_file_nodes(a, b):
   """Compare two nodes' names, contents, and properties, ignoring
-  children.  Return 0 if the same, 1 otherwise.""" 
+  children.  Return 0 if the same, 1 otherwise."""
   if a.name != b.name:
     return 1
   if a.contents != b.contents:
@@ -339,7 +349,7 @@ def handle_dir(path, current_parent, load_props, ignore_svn):
   all_files = os.listdir(path)
   files = []
   dirs = []
-  
+
   # put dirs and files in their own lists, and remove SVN dirs
   for f in all_files:
     f = os.path.join(path, f)
@@ -347,7 +357,7 @@ def handle_dir(path, current_parent, load_props, ignore_svn):
       dirs.append(f)
     elif os.path.isfile(f):
       files.append(f)
-      
+
   # add each file as a child of CURRENT_PARENT
   for f in files:
     fcontents = get_text(f)
@@ -357,7 +367,7 @@ def handle_dir(path, current_parent, load_props, ignore_svn):
       fprops = {}
     current_parent.add_child(SVNTreeNode(os.path.basename(f), None,
                                          fcontents, fprops))
-    
+
   # for each subdir, create a node, walk its tree, add it as a child
   for d in dirs:
     if load_props:
@@ -383,13 +393,13 @@ def get_child(node, name):
 # Helpers for compare_trees
 def default_singleton_handler_a(a, baton):
   "Printing SVNTreeNode A's name, then raise SVNTreeUnequal."
-  print "Got singleton from actual tree:", a.name
+  print "Couldn't find node '%s' in expected tree" % a.name
   a.pprint()
   raise SVNTreeUnequal
 
 def default_singleton_handler_b(b, baton):
   "Printing SVNTreeNode B's name, then raise SVNTreeUnequal."
-  print "Got singleton from expected tree:", b.name
+  print "Couldn't find node '%s' in actual tree" % b.name
   b.pprint()
   raise SVNTreeUnequal
 
@@ -399,17 +409,17 @@ def default_singleton_handler_b(b, baton):
 # EXPORTED ROUTINES ARE BELOW
 
 
-# Main tree comparison routine!  
+# Main tree comparison routine!
 
 def compare_trees(a, b,
                   singleton_handler_a = None,
                   a_baton = None,
                   singleton_handler_b = None,
                   b_baton = None):
-  """Compare SVNTreeNodes A and B, expressing differences using FUNC_A
-  and FUNC_B.  FUNC_A and FUNC_B are functions of two arguments (a
-  SVNTreeNode and a context baton), and may raise exception
-  SVNTreeUnequal.  Their return value is ignored.
+  """Compare SVNTreeNodes A (actual) and B (expected), expressing
+  differences using FUNC_A and FUNC_B.  FUNC_A and FUNC_B are
+  functions of two arguments (a SVNTreeNode and a context baton), and
+  may raise exception SVNTreeUnequal.  Their return value is ignored.
 
   If A and B are both files, then return if their contents,
   properties, and names are all the same; else raise a SVNTreeUnequal.
@@ -541,9 +551,9 @@ def dump_tree(n,indent=""):
 
 def build_generic_tree(nodelist):
   "Given a list of lists of a specific format, return a tree."
-  
+
   root = SVNTreeNode(root_node_name)
-  
+
   for list in nodelist:
     new_branch = create_from_path(list[0], list[1], list[2], list[3])
     root.add_child(new_branch)
@@ -557,22 +567,23 @@ def build_generic_tree(nodelist):
 
 # Parse co/up output into a tree.
 #
-#   Tree nodes will contain no contents, and only one 'status' att.
+#   Tree nodes will contain no contents, a 'status' att, and a
+#   'writelocked' att.
 
 def build_tree_from_checkout(lines):
   "Return a tree derived by parsing the output LINES from 'co' or 'up'."
-  
+
   root = SVNTreeNode(root_node_name)
-  rm1 = re.compile ('^([MAGCUD_ ][MAGCUD_ ])\s+(.+)')
+  rm1 = re.compile ('^([MAGCUD_ ][MAGCUD_ ])([B ])\s+(.+)')
   # There may be other verbs we need to match, in addition to
   # "Restored".  If so, add them as alternatives in the first match
   # group below.
   rm2 = re.compile ('^(Restored)\s+\'(.+)\'')
-  
+
   for line in lines:
     match = rm1.search(line)
     if match and match.groups():
-      new_branch = create_from_path(match.group(2), None, {},
+      new_branch = create_from_path(match.group(3), None, {},
                                     {'status' : match.group(1)})
       root.add_child(new_branch)
     else:
@@ -596,7 +607,7 @@ def build_tree_from_commit(lines):
   root = SVNTreeNode(root_node_name)
   rm1 = re.compile ('^(\w+(  \(bin\))?)\s+(.+)')
   rm2 = re.compile ('^Transmitting')
-  
+
   for line in lines:
     match = rm2.search(line)
     if not match:
@@ -613,24 +624,54 @@ def build_tree_from_commit(lines):
 #
 #   Tree nodes will contain no contents, and these atts:
 #
-#          'status', 'wc_rev', 'repos_rev'
-#             ... and possibly 'locked', 'copied', IFF columns non-empty.
-# 
+#          'status', 'wc_rev',
+#             ... and possibly 'locked', 'copied', 'writelocked',
+#             IFF columns non-empty.
+#
 
 def build_tree_from_status(lines):
-  "Return a tree derived by parsing the output LINES from 'st'."
+  "Return a tree derived by parsing the output LINES from 'st -vuq'."
 
   root = SVNTreeNode(root_node_name)
-  rm = re.compile ('^.+\:.+(\d+)')
-  lastline = string.strip(lines.pop())
-  match = rm.search(lastline)
-  if match and match.groups():
-    repos_rev = match.group(1)
-  else:
-    repos_rev = '?'
-    
+
+  # 'status -v' output looks like this:
+  #
+  #      "%c%c%c%c%c%c %c   %6s   %6s %-12s %s\n"
+  #
+  # (Taken from 'print_status' in subversion/svn/status.c.)
+  #
+  # Here are the parameters.  The middle number in parens is the
+  # match.group(), followed by a brief description of the field:
+  #
+  #    - text status           (1)  (single letter)
+  #    - prop status           (1)  (single letter)
+  #    - wc-lockedness flag    (2)  (single letter: "L" or " ")
+  #    - copied flag           (3)  (single letter: "+" or " ")
+  #    - switched flag         (4)  (single letter: "S" or " ")
+  #    - repos lock status     (5)  (single letter: "K", "O", "B", "T", " ")
+  #
+  #    [one space]
+  #
+  #    - out-of-date flag      (6)  (single letter: "*" or " ")
+  #
+  #    [three spaces]
+  #
+  #    - working revision      (7)  (either digits or "-")
+  #
+  #    [one space]
+  #
+  #    - last-changed revision (8)  (either digits or "?")
+  #
+  #    [one space]
+  #
+  #    - last author           (9)  (string of non-whitespace characters)
+  #
+  #    [one space]
+  #
+  #    - path                 (10)  (string of characters until newline)
+
   # Try http://www.wordsmith.org/anagram/anagram.cgi?anagram=ACDRMGU
-  rm = re.compile ('^([!MACDRUG_ ][MACDRUG_ ])(.)(.)(.)  .   [^0-9-]+(\d+|-)( +\S+ +\S+ +)(.+)')
+  rm = re.compile('^([!MACDRUG_ ][MACDRUG_ ])([L ])([+ ])([S ])([KOBT ]) ([* ])   [^0-9-]*(\d+|-|\?) +(\d|-|\?)+ +(\S+) +(.+)')
   for line in lines:
 
     # Quit when we hit an externals status announcement (### someday we can fix
@@ -638,20 +679,21 @@ def build_tree_from_status(lines):
     # data).
     if re.match(r'^Performing', line):
       break
-    
+
     match = rm.search(line)
     if match and match.groups():
-      if match.group(6) != '-': # ignore items that only exist on repos
+      if match.group(9) != '-': # ignore items that only exist on repos
         atthash = {'status' : match.group(1),
-                   'wc_rev' : match.group(5),
-                   'repos_rev' : repos_rev}
+                   'wc_rev' : match.group(7)}
         if match.group(2) != ' ':
           atthash['locked'] = match.group(2)
         if match.group(3) != ' ':
           atthash['copied'] = match.group(3)
         if match.group(4) != ' ':
           atthash['switched'] = match.group(4)
-        new_branch = create_from_path(match.group(7), None, {}, atthash)
+        if match.group(5) != ' ':
+          atthash['writelocked'] = match.group(5)
+        new_branch = create_from_path(match.group(10), None, {}, atthash)
 
       root.add_child(new_branch)
 
@@ -674,6 +716,19 @@ def build_tree_from_skipped(lines):
 
   return root
 
+def build_tree_from_diff_summarize(lines):
+  "Build a tree from output of diff --summarize"
+  root = SVNTreeNode(root_node_name)
+  rm = re.compile ("^([MAD ][M ])     (.+)\n")
+
+  for line in lines:
+    match = rm.search(line)
+    if match and match.groups():
+      new_branch = create_from_path(match.group(2),
+                                    atts={'status': match.group(1)})
+      root.add_child(new_branch)
+
+  return root
 
 ####################################################################
 # Build trees by looking at the working copy
@@ -695,9 +750,9 @@ def build_tree_from_wc(wc_path, load_props=0, ignore_svn=1):
     # if necessary, store the root dir's props in the root node.
     if load_props:
       root.props = get_props(wc_path)
-      
+
     # Walk the tree recursively
-    handle_dir(os.path.normpath(wc_path), root, load_props, ignore_svn) 
+    handle_dir(os.path.normpath(wc_path), root, load_props, ignore_svn)
 
     return root
 
